/* UNITX-PLUS v1.1:

  One-size-fits-all solution to replace the 2x16 character display that is needed to controle the Uni-TX's SP5055
  Designed by Edwin PE1RZW with thanks to Rob PE1ITR and Rene PA1RKT for their contributions.
  Special thanks to Sjef PE5PVB for the inspiration and his TEF6686 projects.


  Setup:
  Factory default to clean EEPROM values : Hold button 4 while applying DC power to board.
  !Before you can set the offset, band edges and home frecuency: set multipier 'off' in menu!

  Manual:
  Function 1 SHORT PRESS:       Recall memory 1
  Function 1 LONG PRESS:        Save freq to memory 1
  Function 2 SHORT PRESS:       Recall memory 1
  Function 2 LONG PRESS:        Save freq to memory 2
  Function 3 SHORT PRESS:       Toggle nicam on/off
  Function 3 LONG PRESS:        Toggle display on/off and lock other functions
  Function 4 SHORT PRESS:       Toggle antenne (band) A/B
  Function 4 LONG PRESS         Factory default : Hold button 4 while applying DC power to board.

  TX button LONG PRESS:         TXon (Toggle ant-A or ant-B is disabled)
  TX button SHORT PRESS:        TXoff

  Rotary button SHORT PRESS:    Recall home frequency
  Rotary button LONG PRESS:     Enter, save and exit menu's


  Voorlopig werkbare versie.
  Nog doen:
  Wifi/webcontroler realiseren
  Lock detectie met eventuele gevolgen voor TX on/off schakelen.
  Power sensor uitlezing voor beide antennes


  Voorlopig werkbare versie.
  Nog doen:
  INCREMENT_FAST issue in menu oplossen
  Interrupt detectie voor INCREMENT_FAST omzetten naar micros(); voor soepeler tunen
  Wifi/webcontroler realiseren
  Lock detectie met eventuele gevolgen voor TX on/off schakelen.
  Power sensor uitlezing voor beide antennes

  Use this settings in TFT_eSPI User_Setup.h

  #define ILI9341_DRIVER       // Generic driver for common displays
  #define TFT_MISO 19
  #define TFT_MOSI 23
  #define TFT_SCLK 18
  #define TFT_CS   5          // Chip select control pin
  #define TFT_DC   17         // Data Command control pin
  #define TFT_RST  16         // Reset pin (could connect to RST pin)
  #define TOUCH_CS 15         // Chip select pin (T_CS) of touch screen
  #define LOAD_GLCD           // Font 1. Original Adafruit 8 pixel font needs ~1820 bytes in FLASH
  #define LOAD_FONT2          // Font 2. Small 16 pixel high font, needs ~3534 bytes in FLASH, 96 characters
  #define LOAD_FONT4          // Font 4. Medium 26 pixel high font, needs ~5848 bytes in FLASH, 96 characters
  #define LOAD_FONT6          // Font 6. Large 48 pixel font, needs ~2666 bytes in FLASH, only characters 1234567890:-.apm
  #define LOAD_FONT7          // Font 7. 7 segment 48 pixel font, needs ~2438 bytes in FLASH, only characters 1234567890:-.
  #define LOAD_FONT8          // Font 8. Large 75 pixel font needs ~3256 bytes in FLASH, only characters 1234567890:-.
  #define LOAD_GFXFF          // FreeFonts. Include access to the 48 Adafruit_GFX free fonts FF1 to FF48 and custom fonts
  #define SMOOTH_FONT
  #define SPI_FREQUENCY  20000000
  #define SPI_READ_FREQUENCY  20000000
  #define SPI_TOUCH_FREQUENCY  2500000


  ALL OTHER SETTINGS SHOULD BE REMARKED!!!!(hashed out)


*/
#define VERSION "v1.1"


#define NOTEA   "Space for Ant-A label"  //Space for Ant-A label
#define NOTEB   "Space for Ant-B label"  //Space for Ant-A label
#define f3Name  " Rel 2" //F3 relais 2 name
#define f3Stat0  "Off" //F3 relais 2 status off
#define f3Stat1  " On " //F3 relais status on
#define f4Name  "Rel 3" //F4 relais 3 name
#define f4Stat0  "Off" //F4 relais 3 status off//
#define f4Stat1  " On " //F4 relais 3 status on





#include <Arduino.h>
#include <EEPROM.h>
#include <Wire.h>
#include <SPI.h>
#include <analogWrite.h>          // https://github.com/ERROPiX/ESP32_AnalogWrite
#include <TFT_eSPI.h>             // https://github.com/Bodmer/TFT_eSPI
#include <WiFi.h>

#define ENC_DATA_PIN        34     //Rotary encoder data pin
#define ENC_CLK_PIN         36     //Rotary encoder clock pin
#define Button1             39     //Encoder push button
#define Button2             26     //TX push button
#define Button3             25     //Function 1 push button
#define Button4             33     //Function 2 push button
#define Button5             32     //Function 3 push button
#define Button6             35     //Function 4 push button
#define Sensor1             14     //(Future) Power sensor input1. max 3.3v 
#define Sensor2             27     //(Future) Power sensor input2. max 3.3v
#define CONTRASTPIN         2      //TFT contrast pin
#define Relais2             12     //Relais to switch nicam rf of dc on/off
#define Relais3             4      //Relais to switch coaxial relais for ant-a / ant-b scenario's
#define TXPin               13     //State is HIGH when TXon
#define ROTARY_FAST         3      // [ms] rotary speed threshold.  (when increment_fast = 1 this is disbled       
#define INCREMENT_SLOW      1      // Number of increments when dailed slow
#define INCREMENT_FAST      1      // (Future) Number of increments when dailed fast
#define TFT_GREYOUT         0x38E7

//v1.1r: Limits for VCO and BAND (in KHz)
#define VCO_LOW             0
#define VCO_HIGH            3000000  // Max 2625000,witch is the max for sp5055


byte incrementStep;

long setFreq, oldSetFreq;            // Freq voor PLL (UniTX) in KHz

byte enc_clk, enc_clk_old;
volatile int8_t increment;          // Changed by ISR for rotary encoder

int button1StatePrevious;                      // previousstate enc switch
int button2StatePrevious;                      // previousstate TX switch
int button3StatePrevious;                      // previousstate function 1 switch
int button4StatePrevious;                      // previousstate function 2 switch
int button5StatePrevious;                      // previousstate function 3 switch
int button6StatePrevious;                      // previousstate function 4 switch

unsigned long minButtonLongPressDuration = 1000;     // Globale longpress timer

unsigned long previousButton1Millis;                 // Timestamp of the latest reading enc switch
unsigned long button1LongPressMillis;                // Time in ms when we the enc button was pressed
bool button1StateLongPress = false;                  // True if it is a long press of enc button

unsigned long previousButton2Millis;                 // Timestamp of the latest reading TX switch
unsigned long button2LongPressMillis;                // Time in ms when we the TX button was pressed
bool button2StateLongPress = false;                  // True if it is a long press of TX button

unsigned long previousButton3Millis;                 // Timestamp of the latest reading function 1 button
unsigned long button3LongPressMillis;                // Time in ms when we the funtion 1 button was pressed
bool button3StateLongPress = false;                  // True if it is a long press of function 1 button

unsigned long previousButton4Millis;                 // Timestamp of the latest readingfunction 2 button
unsigned long button4LongPressMillis;                // Time in ms when we the funtion 2 button was pressed
bool button4StateLongPress = false;                  // True if it is a long press of function 2 button

unsigned long previousButton5Millis;                 // Timestamp of the latest reading function 3 button
unsigned long button5LongPressMillis;                // Time in ms when we the funtion 3 button was pressed
bool button5StateLongPress = false;                  // True if it is a long press of function 3 button

unsigned long previousButton6Millis;                 // Timestamp of the latest reading function 4 button
unsigned long button6LongPressMillis;                // Time in ms when we the funtion 4 button was pressed
bool button6StateLongPress = false;                  // True if it is a long press of function 4 button

const int intervalButton = 50;                      // Time between two readings of the button state


unsigned long button1PressDuration;                  // Time the button1 is pressed in ms
unsigned long button2PressDuration;                  // Time the button2 is pressed in ms
unsigned long button3PressDuration;                  // Time the button3 is pressed in ms
unsigned long button4PressDuration;                  // Time the button4 is pressed in ms
unsigned long button5PressDuration;                  // Time the button5 is pressed in ms
unsigned long button6PressDuration;                  // Time the button6 is pressed in ms

unsigned long currentMillis;                         // Variabele to store the number of milleseconds since the Arduino has started


int TXPOWER;
int TXPOWERON;
int ConverterSet;
long LowEdgeSet;
long HighEdgeSet;
int ContrastSet;
long HomeSet;
long setMem1;
long setMem2;
long setMem1_A;
long setMem1_B;
long setMem2_A;
long setMem2_B;
long setConverter_A;
long setConverter_B;
long setLowEdge_A;
long setLowEdge_B;
long setHighEdge_A;
long setHighEdge_B;
int TXPOWER_A;
int TXPOWER_B;
long setHome_A;
long setHome_B;
long setFreq_A;
long setFreq_B;

int STEPSIZE_KHZ;
int STEPSIZE_KHZ_A;
int STEPSIZE_KHZ_B;
int menuoption = 30;
int multiplier = 110;
int usermode = 110;
int MPFactor;
int MPFactor_A;
int MPFactor_B;
int AntA;
int Nicam;
int TXState;
int singleMode;


String PowerString;
String ConverterString;
String LowEdgeString;
String HighEdgeString;
String ContrastString;
String HomeString;
String Mem1String;
String Mem2String;
String ShowFreqString;
String MultiplierString;
String singleModeString;


bool menu;
bool menuopen = false;
bool menuMulti = false;
bool menuMode = false;
bool Display = true;


TFT_eSPI tft = TFT_eSPI(240, 320);


void setup()
{
  Serial.begin (115200);

  currentMillis = millis();


  EEPROM.begin(107);
  TXPOWER = EEPROM.read(0);
  ConverterSet = EEPROM.readInt(1);
  LowEdgeSet = EEPROM.readInt(5);
  HighEdgeSet = EEPROM.readInt(9);
  ContrastSet = EEPROM.read(13);
  HomeSet = EEPROM.readInt(14);
  setMem1 = EEPROM.readInt(18);
  setMem2 = EEPROM.readInt(22);
  Nicam = EEPROM.read(26);
  AntA = EEPROM.read(27);
  TXState = EEPROM.read(28);
  setConverter_A = EEPROM.readInt(29);
  setConverter_B = EEPROM.readInt(33);
  setLowEdge_A = EEPROM.readInt(37);
  setLowEdge_B = EEPROM.readInt(41);
  setHighEdge_A = EEPROM.readInt(45);
  setHighEdge_B = EEPROM.readInt(49);
  TXPOWER_A = EEPROM.read(53);
  TXPOWER_B = EEPROM.read(54);
  setHome_A = EEPROM.readInt(55);
  setHome_B = EEPROM.readInt(59);
  setMem1_A = EEPROM.readInt(63);
  setMem1_B = EEPROM.readInt(67);
  setMem2_A = EEPROM.readInt(71);
  setMem2_B = EEPROM.readInt(75);
  setFreq_A = EEPROM.readInt(79);
  setFreq_B = EEPROM.readInt(83);
  setFreq = EEPROM.readInt(87);
  MPFactor = EEPROM.read (91);
  MPFactor_A = EEPROM.read (92);
  MPFactor_B = EEPROM.read (93);
  STEPSIZE_KHZ = EEPROM.readInt (94);
  STEPSIZE_KHZ_A = EEPROM.readInt (98);
  STEPSIZE_KHZ_B = EEPROM.readInt (102);
  singleMode = EEPROM.read(106);


  if ((MPFactor <= 1) || (MPFactor >= 9)) {
    MPFactor = 1;
  }
  setFreqBoot();

  pinMode (TXPin, OUTPUT);
  pinMode (Relais2, OUTPUT);
  pinMode (Relais3, OUTPUT);

  // button input pins

  pinMode (Button1, INPUT);
  pinMode (Button2, INPUT);
  pinMode (Button3, INPUT);
  pinMode (Button4, INPUT);
  pinMode (Button5, INPUT);
  pinMode (Button6, INPUT);




  //v1.2r: setup rotary encoder including 'increment' volatile byte
  // - Also setup interrupts to isrRotaryEncoder isr handler
  // - NOTE: experimental 'RISING' parameter on CLK pin (as used in ISR handler), better?
  increment = 0;
  pinMode (ENC_CLK_PIN, INPUT);
  pinMode (ENC_DATA_PIN, INPUT);
  enc_clk_old = digitalRead(ENC_CLK_PIN);
  attachInterrupt(digitalPinToInterrupt(ENC_DATA_PIN), isrRotaryEncoder, CHANGE);
  attachInterrupt(digitalPinToInterrupt(ENC_CLK_PIN), isrRotaryEncoder, RISING);

  tft.init();
  tft.setRotation(3);
  pinMode (CONTRASTPIN, OUTPUT);



  BuildDisplay();
  ShowPowerLevel();

  analogWrite(CONTRASTPIN, ContrastSet * 2 + 55);

  if (digitalRead(Button6) == LOW) {
    resetEEPROM();
  }


}

void loop()
{

  encSetFrequency();
  currentMillis = millis();    // store the current time in ms
  readButton1State();           // read the button1 state
  readButton2State();           // read the button2 state
  readButton3State();           // read the button3 state
  readButton4State();           // read the button4 state
  readButton5State();           // read the button5 state
  readButton6State();           // read the button6 state
  //getLockState ();          // read phaselock flag from SP5055
  changeMenuoption();
  setMenuValues ();
  setMultiplierOptions();
  setUserMode ();

}


//v1.2r: new ISR for rotary encoder, uses 'increment' variable
// and captures the speed of the rotation (thisInterrupt - lastInterrupt)
//
// NOTE: if this ISR becomes too slow/complex, we might restrict it to just
// read enc_clk and the data pin, read the millis() and let the main loop
// figure out what to do with the results (probably in a function in this same module)
void isrRotaryEncoder()
{


  static long lastInterrupt, thisInterrupt; // static vars because we need to remember them


  lastInterrupt = thisInterrupt;            //save last time we entered this interrupt
  thisInterrupt = millis();                 //record the current moment of entrance

  if ( (thisInterrupt - lastInterrupt) > ROTARY_FAST ) //Speed detection -> fast or slow increments
    incrementStep = INCREMENT_SLOW;
  else
    incrementStep = INCREMENT_FAST;

  enc_clk = digitalRead(ENC_CLK_PIN);
  if ((enc_clk == 1) && (enc_clk_old == 0)) // 0->1 transition
  {
    if (digitalRead(ENC_DATA_PIN) == 1)     // Rotary turned clockwise
      increment += incrementStep;           // so the increment is positive
    else
      increment -= incrementStep;           // else anti-clockwise, decreasing the increment
  }
  enc_clk_old = enc_clk;

}


void encSetFrequency() {

  bool outOfLimits;
  static unsigned long lastSetFreqUpdateTime = 0; // variabele om de vorige keer dat setFreq werd gewijzigd op te slaan

  if (Display && !menu && !menuopen) {
    if (increment != 0)
    {
      oldSetFreq = setFreq;                       //Might be needed later to 'hit the limit'
      setFreq += increment * STEPSIZE_KHZ;
      increment = 0;

      // v1.1r: check limits
      outOfLimits = checkFreqLimits( setFreq, ConverterSet, VCO_LOW, VCO_HIGH, LowEdgeSet, HighEdgeSet );
      if ( !outOfLimits )                           //v1.2r: only checking with limits is necessary
      {
        sendFreqToUniTX( setFreq );
        showNewFreq();

      }
      else setFreq = oldSetFreq;                    //v1.1r: else 'hit the limit'

      Serial.println(setFreq);//For testing purposes, comment afterwards
      //Serial.println(incrementStep);
      showVFO();


    }
  }
}


void changeMenuoption() {

  if ((menu == true && menuopen == false && menuMulti == false && menuMode == false) && increment != 0) {
    if (increment == +1) {
      tft.drawRoundRect(10, menuoption, 300, 18, 5, TFT_BLACK);
      menuoption += 20;
      if (menuoption > 210) {
        menuoption = 30;
      }
      tft.drawRoundRect(10, menuoption, 300, 18, 5, TFT_WHITE);

    } else {
      tft.drawRoundRect(10, menuoption, 300, 18, 5, TFT_BLACK);
      menuoption -= 20;
      if (menuoption < 30) {
        menuoption = 210;
      }
      tft.drawRoundRect(10, menuoption, 300, 18, 5, TFT_WHITE);
    }
    increment = 0;
  }
}

void setUserMode() {

  if (menu == true && menuopen == true && menuMode == true)  {
    menuMode = false;
    switch (usermode) {
      case 110:

        singleMode = 1;

        break;

      case 130:
        singleMode = 0;

        break;

    }
  }
}

void setMultiplierOptions() {

  if (menu == true && menuopen == true && menuMulti == true)  {
    menuMulti = false;
    switch (multiplier) {
      case 110:

        MPFactor = 1;
        STEPSIZE_KHZ = 1000;
        break;

      case 130:
        MPFactor = 2;
        STEPSIZE_KHZ = 500;
        break;



      case 150:
        MPFactor = 4;
        STEPSIZE_KHZ = 250;
        break;


      case 170:
        MPFactor = 8;
        STEPSIZE_KHZ = 125;
        break;

    }
  }
}


void setMenuValues () {
  if (menu == true && menuopen == true)  {
    switch (menuoption) {

      case 30:



        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("RF Power", 150, 70, 4);
        tft.drawString("%", 170, 110, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawRightString(PowerString, 165, 110, 4);

        if (increment != 0) {

          tft.setTextColor(TFT_BLACK);
          tft.drawRightString(PowerString, 165, 110, 4);
          TXPOWER += increment;
          increment = 0;
          if (TXPOWER > 100) {
            TXPOWER = 100;
          } else if (TXPOWER < 0) {
            TXPOWER = 0;
          }

          PowerString = String(TXPOWER, DEC);
          TXPOWERON = TXPOWER * 2 + 12;
          if (TXState == 1) {
            analogWrite(TXPin, TXPOWERON);
          } else
            analogWrite(TXPin, 0);
        }


        break;

      case 50:


        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("Offset:", 150, 70, 4);
        tft.drawString("MHz", 170, 110, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawRightString(ConverterString, 165, 110, 4);
        if (increment != 0) {
          tft.setTextColor(TFT_BLACK);
          tft.drawRightString(ConverterString, 165, 110, 4);
          ConverterSet += increment * STEPSIZE_KHZ;
          increment = 0;
          if (ConverterSet < 0) {
            ConverterSet = 10000000;
          } if (ConverterSet > 10000000) {
            ConverterSet = 0;
          }
          tft.setTextColor(TFT_YELLOW);
          ConverterString = String((ConverterSet * MPFactor) / 1000);
          tft.drawRightString(ConverterString, 165, 110, 4);
          Serial.println(MPFactor);
        }

        break;

      case 70:



        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("Low bandedge:", 150, 70, 4);
        tft.drawString("MHz", 170, 110, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawRightString(LowEdgeString, 165, 110, 4);

        if (increment != 0) {
          tft.setTextColor(TFT_BLACK);
          tft.drawRightString(LowEdgeString, 165, 110, 4);
          LowEdgeSet += increment * STEPSIZE_KHZ;
          increment = 0;
          if (LowEdgeSet < 0) {
            LowEdgeSet = 0;
          }
          tft.setTextColor(TFT_YELLOW);
          LowEdgeString = String (((LowEdgeSet + ConverterSet) * MPFactor) / 1000);
          tft.drawRightString(LowEdgeString, 165, 110, 4);

        }
        break;

      case 90:



        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("High bandedge:", 150, 70, 4);
        tft.drawString("MHz", 170, 110, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawRightString(HighEdgeString, 165, 110, 4);

        if (increment != 0) {
          tft.setTextColor(TFT_BLACK);
          tft.drawRightString(HighEdgeString, 165, 110, 4);
          HighEdgeSet += increment * STEPSIZE_KHZ ;
          increment = 0;
          if (HighEdgeSet < 0) {
            HighEdgeSet = LowEdgeSet;
          }
          tft.setTextColor(TFT_YELLOW);
          HighEdgeString = String (((HighEdgeSet + ConverterSet) * MPFactor) / 1000);
          tft.drawRightString(HighEdgeString, 165, 110, 4);

        }


        break;


      case 110:


        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("Contrast:", 150, 70, 4);
        tft.drawString("%", 170, 110, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawRightString(ContrastString, 165, 110, 4);

        if (increment != 0) {
          tft.setTextColor(TFT_BLACK);
          tft.drawRightString(ContrastString, 165, 110, 4);
          ContrastSet += increment;
          increment = 0;
          if (ContrastSet > 100) {
            ContrastSet = 100;
          } else if (ContrastSet <= 0) {
            ContrastSet = 0;
          }

          tft.setTextColor(TFT_YELLOW);
          ContrastString = String(ContrastSet, DEC);
          tft.drawRightString(ContrastString, 165, 110, 4);
          analogWrite(CONTRASTPIN, ContrastSet * 2 + 55);
        }

        break;

      case 130:


        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("Home Frequency:", 150, 70, 4);
        tft.drawString("MHz", 170, 110, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawRightString(HomeString, 165, 110, 4);
        if (increment != 0) {
          tft.setTextColor(TFT_BLACK);
          tft.drawRightString(HomeString, 165, 110, 4);
          HomeSet += increment * STEPSIZE_KHZ;
          increment = 0;
          if (HomeSet < 0 ) {
            HomeSet = LowEdgeSet;
          }
          tft.setTextColor(TFT_YELLOW);
          HomeString = String (((HomeSet + ConverterSet) * MPFactor) / 1000);
          tft.drawRightString(HomeString, 165, 110, 4);

        }

        break;

      case 150:

        tft.drawRoundRect(125, multiplier, 50, 18, 5, TFT_WHITE);
        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("Multiplier", 150, 70, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawCentreString("Off", 150, 110, 2);
        tft.drawCentreString("x 2", 150, 130, 2);
        tft.drawCentreString("x 4", 150, 150, 2);
        tft.drawCentreString("x 8", 150, 170, 2);
        menuMulti = true;

        if (increment != 0) {
          if (increment == +1) {
            tft.drawRoundRect(125, multiplier, 50, 18, 5, TFT_BLACK);
            multiplier += 20;
            if (multiplier > 170) {
              multiplier = 110;
            }
            tft.drawRoundRect(125, multiplier, 50, 18, 5, TFT_WHITE);

          } else {
            tft.drawRoundRect(125, multiplier, 50, 18, 5, TFT_BLACK);
            multiplier -= 20;
            if (multiplier < 110) {
              multiplier = 170;
            }
            tft.drawRoundRect(125, multiplier, 50, 18, 5, TFT_WHITE);
          }
          increment = 0;
        }

        break;

      case 210:

        tft.drawRoundRect(125, usermode, 50, 18, 5, TFT_WHITE);
        tft.setTextColor(TFT_WHITE);
        tft.drawCentreString("MultiMode", 150, 70, 4);
        tft.setTextColor(TFT_YELLOW);
        tft.drawCentreString("Off", 150, 110, 2);
        tft.drawCentreString("On", 150, 130, 2);
        menuMode = true;

        if (increment != 0) {
          if (increment == +1) {
            tft.drawRoundRect(125, usermode, 50, 18, 5, TFT_BLACK);
            usermode += 20;
            if (usermode > 130) {
              usermode = 110;
            }
            tft.drawRoundRect(125, usermode, 50, 18, 5, TFT_WHITE);

          } else {
            tft.drawRoundRect(125, usermode, 50, 18, 5, TFT_BLACK);
            usermode -= 20;
            if (usermode < 110) {
              usermode = 130;
            }
            tft.drawRoundRect(125, usermode, 50, 18, 5, TFT_WHITE);
          }
          increment = 0;
        }
    }
  }
}

// showNewFreq displays the frequency (with offset)
// v1.1r: setTextColor now called with background color - no flickering?
void showNewFreq()
{

  tft.setTextColor(TFT_YELLOW, TFT_BLACK);
  ShowFreqString = String (((setFreq + ConverterSet) * MPFactor) / 1000);
  tft.drawRightString(ShowFreqString, 240, 45, 7);
  if ((((setFreq + ConverterSet)*MPFactor) / 1000) <= 999) {
    tft.setTextColor(TFT_BLACK);
    tft.drawString("8", 112, 45, 7);
  } if ((((setFreq + ConverterSet)*MPFactor) / 1000) <= 9999) {
    tft.setTextColor(TFT_BLACK);
    tft.drawString("8", 80, 45, 7);
  }

}

void showNewTXPOWER()
{
  tft.setTextColor(TFT_WHITE);
  tft.drawCentreString("RF power", 150, 70, 4);
  tft.drawString("%", 170, 110, 4);
  tft.setTextColor(TFT_YELLOW, TFT_BLACK);
  tft.drawRightString(PowerString, 165, 110, 4);
}

// v1.1r: check bandlimits and vco limits. If one of these limits is hit, returns TRUE, otherwise FALSE
bool checkFreqLimits( long pllFreq, long offsetFreq, long vcoLow, long vcoHigh, long bandLow, long bandHigh )
{
  bool outOfLimits = 0;
  long bandFreq;

  bandFreq = pllFreq + offsetFreq;

  if ( (pllFreq < vcoLow)  ||
       (pllFreq > vcoHigh) ||
       (bandFreq < bandLow + ConverterSet)  ||
       (bandFreq > bandHigh + ConverterSet))
  {
    outOfLimits = 1;
  }

  return outOfLimits;
}

void sendFreqToUniTX( long freq )
{
  unsigned char byte2, byte3;
  unsigned int divider;
  unsigned long xtal = 4000;

  // Tweede deel code van Rob PE1ITR==================================================================================
  Wire.begin();
  divider = (freq * 32L) / xtal;
  byte2 = (divider >> 8) & 0x7F;      //high byte
  byte3 = divider & 0xff;             //low byte

  Wire.beginTransmission(0x61);
  int rbyte2 = Wire.write(byte2);     //byte2 of datasheet
  int rbyte3 = Wire.write(byte3);     //byte3
  int rbyte4 = Wire.write(B11001110); // byte4
  int rbyte5 = Wire.write(B11001110); //byte5

  int r = Wire.endTransmission();
  //=====================================================================================================================
}

/*
  void recieveLockState()
  {
  unsigned char flag;
  Wire.beginTransmission (0x61);
  int lockByte = Wire.read (flag);
  if (lockByte == 1) {
    tft.drawRoundRect(266, 34, 51, 18, 2, TFT_WHITE); //LOCKED
    tft.setTextColor(TFT_WHITE);
    tft.drawString("LOCKED", 269, 35, 2);
  } else {
    tft.drawRoundRect(266, 34, 51, 18, 2, TFT_GREYOUT); //not LOCKED
    tft.setTextColor(TFT_GREYOUT);
    tft.drawString("LOCKED", 269, 35, 2);
  }
  }
*/


void BuildDisplay()
{


  Mem1String =  (((ConverterSet + setMem1) * MPFactor) / 1000);
  Mem2String =  (((ConverterSet + setMem2) * MPFactor) / 1000);
  PowerString = TXPOWER;

  tft.fillScreen(TFT_BLACK);
  tft.drawRect(1, 1, 319, 239, TFT_SKYBLUE);
  tft.drawLine(1, 30, 319, 30, TFT_SKYBLUE);
  tft.drawLine(1, 100, 319, 100, TFT_SKYBLUE);
  tft.drawLine(1, 130, 319, 130, TFT_SKYBLUE);
  tft.drawLine(1, 160, 319, 160, TFT_SKYBLUE);
  tft.drawLine(160, 1, 160, 30, TFT_SKYBLUE);
  tft.drawLine(270, 1, 270, 30, TFT_SKYBLUE);
  tft.drawRoundRect(243, 176, 73, 57, 1, TFT_WHITE); //Ant rect
  tft.fillRoundRect(245, 177, 70, 55, 1, TFT_BLUE); // Ant fill
  tft.setTextColor(TFT_WHITE, TFT_BLUE);
  tft.drawCentreString(f4Name, 278, 182, 4);
  tft.setTextColor(TFT_RED);
  tft.drawString("Uni-TX+", 170, 8, 4);
  tft.setTextColor(TFT_YELLOW);
  tft.drawString("MHz", 256, 64, 4);
  tft.drawRoundRect(6, 66, 40, 16, 2, TFT_GREYOUT); // Home
  tft.drawRoundRect(6, 83, 40, 16, 2, TFT_WHITE); // Vfo
  tft.drawRoundRect(48, 66, 23, 16, 2, TFT_GREYOUT); // M1
  tft.drawRoundRect(48, 83, 23, 16, 2, TFT_GREYOUT); // M2
  tft.drawRoundRect(266, 34, 51, 18, 2, TFT_GREYOUT); //not LOCKED


  tft.setTextColor(TFT_GREYOUT);
  tft.drawString("LOCKED", 269, 35, 2);
  tft.drawString("M1", 52, 66, 2);
  tft.drawString("M2", 52, 83, 2);
  tft.drawString("HOME", 10, 66, 2);
  tft.setTextColor(TFT_WHITE);
  tft.drawString("VFO", 14, 83, 2);
  tft.drawString(VERSION, 280, 10, 2);
  tft.drawRoundRect(5, 176, 73, 57, 2, TFT_WHITE); //M1 rect
  tft.fillRoundRect(6, 177, 71, 55, 1, TFT_BLUE); //M1 fill
  tft.drawString("M1", 25, 182, 4);  // Memory 1
  tft.drawRoundRect(81, 176, 73, 57, 2, TFT_WHITE); //M2 rect
  tft.fillRoundRect(82, 177, 71, 55, 1, TFT_BLUE); //M2 fill
  tft.drawString("M2", 100, 182, 4);  // Memory 2
  tft.drawRoundRect(157, 176, 83, 57, 1, TFT_WHITE);//Nicam rect
  tft.fillRoundRect(158, 177, 81, 55, 1, TFT_BLUE); //Nicam fill
  tft.setTextColor(TFT_WHITE, TFT_BLUE);
  tft.drawCentreString(f3Name, 198, 182, 4);
  showNewFreq();
  antStatus();
  nicamStatus();
  txStatus();
  sendFreqToUniTX( setFreq );



}

void BuildMenu()
{

  PowerString = TXPOWER;
  ConverterString = (ConverterSet / 1000);
  LowEdgeString = (((LowEdgeSet + ConverterSet) * MPFactor) / 1000);
  HighEdgeString = (((HighEdgeSet + ConverterSet) * MPFactor) / 1000);
  ContrastString = ContrastSet;
  HomeString = (((HomeSet + ConverterSet) * MPFactor) / 1000);
  MultiplierString = MPFactor;
  singleModeString = singleMode;

  tft.fillScreen(TFT_BLACK);
  tft.drawRect(1, 1, 319, 239, TFT_WHITE);
  tft.drawRect(2, 2, 318, 238, TFT_WHITE);
  tft.drawLine(1, 20, 319, 20, TFT_WHITE);
  tft.drawLine(1, 21, 319, 21, TFT_WHITE);
  tft.setTextColor(TFT_SKYBLUE);
  tft.drawString("LONG PRESS ENCODER TO EXIT", 20, 4, 2);
  tft.setTextColor(TFT_WHITE);
  tft.drawRightString(VERSION, 305, 4, 2);
  tft.drawRoundRect(10, menuoption, 300, 18, 5, TFT_WHITE);

  tft.setTextColor(TFT_WHITE);
  tft.drawString("Set RF Power", 20, 30, 2);
  tft.drawString("Set Frequency Offset", 20, 50, 2);
  tft.drawString("Set Low Bandedge", 20, 70, 2);
  tft.drawString("Set High Bandedge", 20, 90, 2);
  tft.drawString("Set Contrast Display", 20, 110, 2);
  tft.drawString("Set Home Frequency", 20, 130, 2);
  tft.drawString("Set Multiplier", 20, 150, 2);
  tft.drawString("Set Futere Use", 20, 170, 2);
  tft.drawString("Set Future Use", 20, 190, 2);
  tft.drawString("Set MultiMode", 20, 211, 2);

  tft.setTextColor(TFT_WHITE);
  tft.drawRightString("%", 305, 30, 2);
  tft.drawRightString("MHz", 305, 50, 2);
  tft.drawRightString("MHz", 305, 70, 2);
  tft.drawRightString("MHz", 305, 90, 2);
  tft.drawRightString("%", 305, 110, 2);
  tft.drawRightString("MHz", 305, 130, 2);
  tft.drawRightString("x", 305, 150, 2);
  tft.drawRightString("%", 305, 170, 2);
  tft.drawRightString("%", 305, 190, 2);


  tft.setTextColor(TFT_YELLOW);
  tft.drawRightString(PowerString, 270, 30, 2);
  tft.drawRightString(ConverterString, 270, 50, 2);
  tft.drawRightString(LowEdgeString, 270, 70, 2);
  tft.drawRightString(HighEdgeString, 270, 90, 2);
  tft.drawRightString(ContrastString, 270, 110, 2);
  tft.drawRightString(HomeString, 270, 130, 2);




  if ((MPFactor <= 1) || (MPFactor >= 9)) {
    MPFactor = 1;
    tft.setTextColor(TFT_YELLOW);
    tft.drawRightString("Off", 270, 150, 2);
    tft.setTextColor(TFT_BLACK);
    tft.drawRightString("x", 305, 150, 2);
  } else {
    tft.setTextColor(TFT_YELLOW);
    tft.drawRightString(MultiplierString, 270, 150, 2);
  }

  if ((singleMode <= 0) || (singleMode >= 2)) {
    singleMode = 0;
    tft.setTextColor(TFT_YELLOW, TFT_BLACK);
    tft.drawRightString(" On", 270, 211, 2);

  } else {
    tft.setTextColor(TFT_YELLOW, TFT_BLACK);
    tft.drawRightString("Off", 270, 211, 2);
  }
}


void ShowPowerLevel() {


  tft.setTextColor(TFT_WHITE);
  tft.drawString("Power", 10, 105, 2);
  tft.drawString("%", 256, 105, 2);
  tft.setTextColor(TFT_YELLOW);
  tft.drawString(PowerString, 205, 105, 2);
  if (TXState == 0) {
    TXPOWERON = 0;
  }
  if (TXPOWERON > 55) {
    tft.fillRect(48, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(48, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 75) {
    tft.fillRect(62, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(62, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 95) {
    tft.fillRect(76, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(76, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 115) {
    tft.fillRect(90, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(90, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 125) {
    tft.fillRect(104, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(104, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 145) {
    tft.fillRect(118, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(118, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 165) {
    tft.fillRect(132, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(132, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 185) {
    tft.fillRect(146, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(146, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 205) {
    tft.fillRect(160, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(160, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 217) {
    tft.fillRect(174, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(174, 113, 12, 4, TFT_GREYOUT);
  }
  if (TXPOWERON > 254) {
    tft.fillRect(188, 113, 12, 4, TFT_GREEN);
  } else {
    tft.fillRect(188, 113, 12, 4, TFT_GREYOUT);
  }
}





void readButton1State() {


  if (currentMillis - previousButton1Millis > intervalButton) {
    int button1State = digitalRead(Button1);
    if (button1State == LOW && button1StatePrevious == HIGH && !button1StateLongPress) {
      button1LongPressMillis = currentMillis;
      button1StatePrevious = LOW;
    }
    button1PressDuration = currentMillis - button1LongPressMillis;

    // Als hoofdscherm up is opent een long press op encoder button het menu.

    if ((menu == false) && (button1State == LOW && !button1StateLongPress && button1PressDuration >= minButtonLongPressDuration)) {
      button1StateLongPress = true;
      Serial.println("Menu");
      BuildMenu();
      menu = true;
      if (AntA == 0) {

        setFreq_A = setFreq;
      }
      else if (AntA == 1) {

        setFreq_B = setFreq;


      }

    }

    // Als menu up is opent een long press op encoder button het hoofdscherm.

    if ((menu == true) && (button1State == LOW && !button1StateLongPress && button1PressDuration >= minButtonLongPressDuration)) {
      button1StateLongPress = true;
      Serial.println("Hoofd scherm");

      EEPROM.write(0, TXPOWER);
      EEPROM.writeInt(1, ConverterSet);
      EEPROM.writeInt(5, LowEdgeSet);
      EEPROM.writeInt(9, HighEdgeSet);
      EEPROM.write(13, ContrastSet);
      EEPROM.writeInt(14, HomeSet);
      EEPROM.write(91, MPFactor);
      EEPROM.writeInt(94, STEPSIZE_KHZ);
      EEPROM.write(106, singleMode);


      EEPROM.commit();

      increment = 0;
      menu = false;
      menuopen = false;
      menuMulti = false;
      menuMode = false;
      ShowPowerLevel();
      if (AntA == 0) {
        setConverter_A = ConverterSet;
        setLowEdge_A = LowEdgeSet;
        setHighEdge_A = HighEdgeSet;
        TXPOWER_A = TXPOWER;
        setHome_A = HomeSet;
        //setFreq_A = setFreq;
        setMem1_A = setMem1;
        setMem2_A = setMem2;
        setFreq = setFreq_A;
        MPFactor_A = MPFactor;
        STEPSIZE_KHZ_A = STEPSIZE_KHZ;
        EEPROM.writeInt(29, setConverter_A);
        EEPROM.writeInt(37, setLowEdge_A);
        EEPROM.writeInt(45, setHighEdge_A);
        EEPROM.write(53, TXPOWER_A);
        EEPROM.writeInt(55, setHome_A);
        EEPROM.writeInt(79, setFreq_A);
        EEPROM.write (92, MPFactor_A);
        EEPROM.writeInt(98, STEPSIZE_KHZ_A);
        EEPROM.commit();
        BuildDisplay();
        antStatus();


      }
      else if (AntA == 1) {
        setConverter_B = ConverterSet;
        setLowEdge_B = LowEdgeSet;
        setHighEdge_B = HighEdgeSet;
        TXPOWER_B = TXPOWER;
        setHome_B = HomeSet;
        //setFreq_B = setFreq;
        setMem1_B = setMem1;
        setMem2_B = setMem2;
        setFreq = setFreq_B;
        MPFactor_B = MPFactor;
        STEPSIZE_KHZ_B = STEPSIZE_KHZ;
        EEPROM.writeInt(33, setConverter_B);
        EEPROM.writeInt(41, setLowEdge_B);
        EEPROM.writeInt(49, setHighEdge_B);
        EEPROM.write(54, TXPOWER_B);
        EEPROM.writeInt(59, setHome_B);
        EEPROM.writeInt(83, setFreq_B);
        EEPROM.write (93, MPFactor_B);
        EEPROM.writeInt(102, STEPSIZE_KHZ_B);
        EEPROM.commit();
        BuildDisplay();
        antStatus();


      }


      if ((menu == true) && (menuopen == true) && (button1State == LOW && !button1StateLongPress && button1PressDuration >= minButtonLongPressDuration)) {
        button1StateLongPress = true;
        Serial.println("Terug naar menu");
        increment = 0;
        BuildMenu();
        menu = true;
        menuopen = false;



      }
    }


    if (button1State == HIGH && button1StatePrevious == LOW) {
      button1StatePrevious = HIGH;
      button1StateLongPress = false;


      if ((menu == true && menuopen == false) && (!button1StateLongPress && button1PressDuration < minButtonLongPressDuration)) {
        Serial.println("Select item");
        menuopen = true;
        menu = true;
        tft.drawRoundRect(30, 40, 240, 160, 5, TFT_WHITE);
        tft.fillRoundRect(32, 42, 236, 156, 5, TFT_BLACK);


      }

      else if ((menuopen == true) && (!button1StateLongPress && button1PressDuration < minButtonLongPressDuration)) {
        Serial.println("Terug naar menu");
        menuopen = false;
        menu = true;
        BuildMenu();

      }


      if (menu == false && menuopen == false && currentMillis > 1000) {
        if (!button1StateLongPress && (button1PressDuration < minButtonLongPressDuration)) {
          Serial.println("Recall Home");
          setFreq = HomeSet;
          showNewFreq();
          showHome();
          sendFreqToUniTX( setFreq );
          if (AntA == 0) {

            setFreq_A = setFreq;
            EEPROM.writeInt(79, setFreq_A);
            EEPROM.commit();
          }
          else if (AntA == 1) {

            setFreq_B = setFreq;
            EEPROM.writeInt(83, setFreq_B);
            EEPROM.commit();


          }

        }

      }
    }
    previousButton1Millis = currentMillis;
  }
}

void readButton2State() {


  if (currentMillis - previousButton2Millis > intervalButton)
  {
    int button2State = digitalRead(Button2);
    if (button2State == LOW && button2StatePrevious == HIGH && !button2StateLongPress) {
      button2LongPressMillis = currentMillis;
      button2StatePrevious = LOW;
    }
    button2PressDuration = currentMillis - button2LongPressMillis;

    if (Display == true && TXState <= 0 && menu == false && currentMillis > 1000) {
      if (button2State == LOW && !button2StateLongPress && button2PressDuration >= minButtonLongPressDuration) {
        button2StateLongPress = true;
        TXState = 1;
        EEPROM.write(28, TXState);
        EEPROM.commit();
        txStatus();
        delay(300);                // waits for 300ms to powerup UNITX
        sendFreqToUniTX( setFreq );



      }
    }
    else if (button2State == HIGH && button2StatePrevious == LOW) {
      button2StatePrevious = HIGH;
      button2StateLongPress = false;

      if (Display == true && TXState >= 1 && menu == false && currentMillis > 1000) {
        if (!button2StateLongPress && button2PressDuration < minButtonLongPressDuration) {
          TXState = 0;
          EEPROM.write(28, TXState);
          EEPROM.commit();
          txStatus();

        }
      }
    }


    previousButton2Millis = currentMillis;

  }

}
void readButton3State() {


  if (currentMillis - previousButton3Millis > intervalButton) {
    int button3State = digitalRead(Button3);
    if (button3State == LOW && button3StatePrevious == HIGH && !button3StateLongPress) {
      button3LongPressMillis = currentMillis;
      button3StatePrevious = LOW;
    }
    button3PressDuration = currentMillis - button3LongPressMillis;


    if (Display == true && menu == false && button3State == LOW && !button3StateLongPress && button3PressDuration >= minButtonLongPressDuration) {
      button3StateLongPress = true;
      Serial.println("Save M1");
      setMem1 = setFreq;
      Mem1String =  (((ConverterSet + setMem1) * MPFactor) / 1000);
      tft.setTextColor(TFT_WHITE, TFT_BLUE);
      tft.drawCentreString(Mem1String, 42, 205, 4);
      EEPROM.writeInt(18, setMem1);
      EEPROM.commit();

      if (AntA == 0) {
        setMem1_A = setMem1;
        EEPROM.writeInt(63, setMem1_A);
        EEPROM.commit();
        BuildDisplay();
      } else   {
        setMem1_B = setMem1;
        EEPROM.writeInt(67, setMem1_B);
        EEPROM.commit();
        BuildDisplay();
      }
    }

    if (Display == true && menu == false && menuopen == false && currentMillis > 1000) {
      if (button3State == HIGH && button3StatePrevious == LOW) {
        button3StatePrevious = HIGH;
        button3StateLongPress = false;

        if (button3PressDuration < minButtonLongPressDuration) {
          Serial.println("Recall M1");
          setFreq = setMem1;
          sendFreqToUniTX( setFreq );
          showNewFreq();
          showM1();
          if (AntA == 0) {
            setFreq_A = setFreq;
            EEPROM.writeInt(79, setFreq_A);
            EEPROM.commit();
          } else if (AntA == 1)  {
            setFreq_B = setFreq;
            EEPROM.writeInt(83, setFreq_B);
            EEPROM.commit();
          }
        }
      }
    }


    previousButton3Millis = currentMillis;

  }

}
void readButton4State() {


  if (currentMillis - previousButton4Millis > intervalButton) {
    int button4State = digitalRead(Button4);
    if (button4State == LOW && button4StatePrevious == HIGH && !button4StateLongPress) {
      button4LongPressMillis = currentMillis;
      button4StatePrevious = LOW;
    }
    button4PressDuration = currentMillis - button4LongPressMillis;


    if (Display == true && menu == false && button4State == LOW && !button4StateLongPress && button4PressDuration >= minButtonLongPressDuration) {
      button4StateLongPress = true;
      Serial.println("Save M2");
      setMem2 = setFreq;
      Mem2String =  (((ConverterSet + setMem2) * MPFactor) / 1000);
      tft.setTextColor(TFT_WHITE, TFT_BLUE);
      tft.drawCentreString(Mem2String, 118, 205, 4);
      EEPROM.writeInt(22, setMem2);

      EEPROM.commit();
      if (AntA == 0) {
        setMem2_A = setMem2;
        EEPROM.writeInt(71, setMem2_A);
        EEPROM.commit();
        BuildDisplay();
      } else if (AntA == 1)  {
        setMem2_B = setMem2;
        EEPROM.writeInt(75, setMem2_B);
        EEPROM.commit();
        BuildDisplay();
      }
    }
    if (Display == true && menu == false && menuopen == false && currentMillis > 1000) {
      if (button4State == HIGH && button4StatePrevious == LOW) {
        button4StatePrevious = HIGH;
        button4StateLongPress = false;

        if (button4PressDuration < minButtonLongPressDuration) {
          Serial.println("Recall M2");
          setFreq = setMem2;
          sendFreqToUniTX( setFreq );
          showNewFreq();
          showM2();
          if (AntA == 0) {
            setFreq_A = setFreq;
            EEPROM.writeInt(79, setFreq_A);
            EEPROM.commit();
          } else if (AntA == 1)  {
            setFreq_B = setFreq;
            EEPROM.writeInt(83, setFreq_B);
            EEPROM.commit();
          }
        }
      }
    }

    previousButton4Millis = currentMillis;

  }

}

void readButton5State() {


  if (currentMillis - previousButton5Millis > intervalButton) {
    int button5State = digitalRead(Button5);
    if (button5State == LOW && button5StatePrevious == HIGH && !button5StateLongPress) {
      button5LongPressMillis = currentMillis;
      button5StatePrevious = LOW;
    }
    button5PressDuration = currentMillis - button5LongPressMillis;


    if (menu == false && button5State == LOW && !button5StateLongPress && button5PressDuration >= minButtonLongPressDuration) {
      button5StateLongPress = true;
      Serial.println("Scherm aan/uit");
      if (Display == true) {
        Display = false;
        analogWrite(CONTRASTPIN, 0);
      } else if (Display == false) {
        Display = true;
        increment = 0;
        analogWrite(CONTRASTPIN, ContrastSet * 2 + 55);
      }


    }
    if (menu == false && menuopen == false && currentMillis > 1000) {
      if (button5State == HIGH && button5StatePrevious == LOW) {
        button5StatePrevious = HIGH;
        button5StateLongPress = false;

        if (button5PressDuration < minButtonLongPressDuration && Nicam <= 0 && Display == true) {
          Nicam = 1;


          Serial.println("F3 On");
          tft.setTextColor(TFT_WHITE, TFT_BLUE);
          tft.drawCentreString(" On ", 200, 205, 4);  // Relais 2
          digitalWrite(Relais2, HIGH);
          EEPROM.write(26, Nicam);
          EEPROM.commit();

        } else if (button5PressDuration < minButtonLongPressDuration && Nicam >= 1 && Display == true) {
          Nicam = 0;
          Serial.println("F3 Off");
          tft.setTextColor(TFT_WHITE, TFT_BLUE);
          tft.drawCentreString(" Off ", 200, 205, 4);  // Relais 2
          digitalWrite(Relais2, LOW);
          EEPROM.write(26, Nicam);
          EEPROM.commit();
        }
      }

    }

    previousButton5Millis = currentMillis;

  }

}
void readButton6State() {


  if (currentMillis - previousButton6Millis > intervalButton) {
    int button6State = digitalRead(Button6);
    if (button6State == LOW && button6StatePrevious == HIGH && !button6StateLongPress) {
      button6LongPressMillis = currentMillis;
      button6StatePrevious = LOW;
    }
    button6PressDuration = currentMillis - button6LongPressMillis;


    if (button6State == LOW && !button6StateLongPress && button6PressDuration >= minButtonLongPressDuration) {
      button6StateLongPress = true;
      Serial.println("niet gebruikt");


    }


    if (Display == true && TXState == 0 && menu == false && menuopen == false && singleMode == 0 && currentMillis > 1000) {
      if (button6State == HIGH && button6StatePrevious == LOW) {
        button6StatePrevious = HIGH;
        button6StateLongPress = false;

        if (button6PressDuration < minButtonLongPressDuration && AntA <= 0) {
          AntA = 1;
          Serial.println("Antenne B");
          tft.setTextColor(TFT_WHITE, TFT_BLUE);
          tft.drawCentreString(f4Stat1, 278, 205, 4);  // Relais 3
          digitalWrite(Relais3, HIGH);
          setFreq_A = setFreq;
          setFreq = setFreq_B;
          ConverterSet = setConverter_B;
          LowEdgeSet = setLowEdge_B;
          HighEdgeSet = setHighEdge_B;
          TXPOWER = TXPOWER_B;
          HomeSet = setHome_B;
          setMem1 = setMem1_B;
          setMem2 = setMem2_B;
          MPFactor = MPFactor_B;
          STEPSIZE_KHZ = STEPSIZE_KHZ_B;
          EEPROM.write(0, TXPOWER);
          EEPROM.writeInt(1, ConverterSet);
          EEPROM.writeInt(5, LowEdgeSet);
          EEPROM.writeInt(9, HighEdgeSet);
          EEPROM.writeInt(14, HomeSet);
          EEPROM.write(27, AntA);
          EEPROM.write (91, MPFactor);
          EEPROM.writeInt(94, STEPSIZE_KHZ);
          EEPROM.commit();
          BuildDisplay();
        } else if (button6PressDuration < minButtonLongPressDuration && AntA >= 1) {
          AntA = 0;
          Serial.println("Antenne A");
          tft.setTextColor(TFT_WHITE, TFT_BLUE);
          tft.drawCentreString(f4Stat0, 278, 205, 4);  // Relais 3
          digitalWrite(Relais3, LOW);
          setFreq_B = setFreq;
          setFreq = setFreq_A;
          ConverterSet = setConverter_A;
          LowEdgeSet = setLowEdge_A;
          HighEdgeSet = setHighEdge_A;
          TXPOWER = TXPOWER_A;
          HomeSet = setHome_A;
          setMem1 = setMem1_A;
          setMem2 = setMem2_A;
          MPFactor = MPFactor_A;
          STEPSIZE_KHZ = STEPSIZE_KHZ_A;
          EEPROM.write(0, TXPOWER);
          EEPROM.writeInt(1, ConverterSet);
          EEPROM.writeInt(5, LowEdgeSet);
          EEPROM.writeInt(9, HighEdgeSet);
          EEPROM.writeInt(14, HomeSet);
          EEPROM.write(27, AntA);
          EEPROM.write (91, MPFactor);
          EEPROM.writeInt(94, STEPSIZE_KHZ);
          EEPROM.commit();
          BuildDisplay();

        }

      }
    }

    if (Display == true && menu == false && menuopen == false && singleMode == 1 && currentMillis > 1000) {
      if (button6State == HIGH && button6StatePrevious == LOW) {
        button6StatePrevious = HIGH;
        button6StateLongPress = false;

        if (button6PressDuration < minButtonLongPressDuration && AntA <= 0) {
          AntA = 1;
          Serial.println("F4 On");
          tft.setTextColor(TFT_WHITE, TFT_BLUE);
          tft.drawCentreString(f4Stat1, 278, 205, 4);  // Relais 3
          digitalWrite(Relais3, HIGH);
          EEPROM.write(27, AntA);
          EEPROM.commit();

        } else if (button6PressDuration < minButtonLongPressDuration && AntA >= 1) {
          AntA = 0;
          Serial.println("F4 Off");
          tft.setTextColor(TFT_WHITE, TFT_BLUE);
          tft.drawCentreString(f4Stat0, 278, 205, 4);  // Relais 3
          digitalWrite(Relais3, LOW);
          EEPROM.write(27, AntA);
          EEPROM.commit();


        }

      }
    }
    previousButton6Millis = currentMillis;


  }
}

void resetEEPROM() {

  for (int i = 0; i < 512; i++) {
    EEPROM.write(i, 0);
  }
  EEPROM.commit();
  delay(500);
}



void setFreqBoot() {
  if (AntA >= 1) {

    setFreq = setFreq_B;
    setMem1 = setMem1_B;
    setMem2 = setMem2_B;
  } else if (AntA >= 0) {
    setFreq = setFreq_A;
    setMem1 = setMem1_A;
    setMem2 = setMem2_A;

  }
}




void antStatus() {
  if (AntA >= 1) {
    setMem1 = setMem1_B;
    setMem2 = setMem2_B;
    STEPSIZE_KHZ = STEPSIZE_KHZ_B;
    Serial.println("Ant status B");
    tft.setTextColor(TFT_WHITE);
    tft.drawString(NOTEB, 10, 10, 2);
    tft.setTextColor(TFT_WHITE, TFT_BLUE);
    tft.drawCentreString(f4Stat1, 278, 205, 4);  // Relais 3
    tft.drawCentreString(Mem1String, 42, 205, 4);
    tft.drawCentreString(Mem2String, 118, 205, 4);
    digitalWrite(Relais3, HIGH);
  } else if (AntA <= 0) {
    setMem1 = setMem1_A;
    setMem2 = setMem2_A;
    STEPSIZE_KHZ = STEPSIZE_KHZ_A;
    Serial.println("Ant status A");
    tft.setTextColor(TFT_WHITE);
    tft.drawString(NOTEA, 10, 10, 2);
    tft.setTextColor(TFT_WHITE, TFT_BLUE);
    tft.drawCentreString(f4Stat0, 278, 205, 4);  // Relais 3
    tft.drawCentreString(Mem1String, 42, 205, 4);
    tft.drawCentreString(Mem2String, 118, 205, 4);
    digitalWrite(Relais3, LOW);
  }
}
void nicamStatus() {
  if (Nicam >= 1) {
    Serial.println("Nicam status On");
    tft.setTextColor(TFT_WHITE, TFT_BLUE);
    tft.drawCentreString(f3Stat1, 200, 205, 4);  // Relais 2
    digitalWrite(Relais2, HIGH);
  } else if (Nicam <= 0) {
    Serial.println("Nicam status Off");
    tft.setTextColor(TFT_WHITE, TFT_BLUE);
    tft.drawCentreString(f3Stat0, 200, 205, 4);  // Relais 2
    digitalWrite(Relais2, LOW);
  }
}

void showHome() {

  tft.setTextColor(TFT_WHITE);
  tft.drawString("HOME", 10, 66, 2);
  tft.drawRoundRect(6, 66, 40, 16, 2, TFT_WHITE); // Home
  tft.drawRoundRect(48, 66, 23, 16, 2, TFT_GREYOUT); // M1
  tft.drawRoundRect(48, 83, 23, 16, 2, TFT_GREYOUT); // M2
  tft.drawRoundRect(6, 83, 40, 16, 2, TFT_GREYOUT); // Vfo
  tft.setTextColor(TFT_GREYOUT);
  tft.drawString("M1", 52, 66, 2);
  tft.drawString("M2", 52, 83, 2);
  tft.drawString("VFO", 14, 83, 2);
}

void showM1() {
  tft.drawRoundRect(48, 66, 23, 16, 2, TFT_WHITE); // M1
  tft.setTextColor(TFT_WHITE);
  tft.drawString("M1", 52, 66, 2);
  tft.drawRoundRect(48, 83, 23, 16, 2, TFT_GREYOUT); // M2
  tft.drawRoundRect(6, 66, 40, 16, 2, TFT_GREYOUT); // Home
  tft.drawRoundRect(6, 83, 40, 16, 2, TFT_GREYOUT); // Vfo
  tft.setTextColor(TFT_GREYOUT);
  tft.drawString("M2", 52, 83, 2);
  tft.drawString("HOME", 10, 66, 2);
  tft.drawString("VFO", 14, 83, 2);
}
void showM2() {
  tft.drawRoundRect(6, 66, 40, 16, 2, TFT_GREYOUT); // Home
  tft.drawRoundRect(6, 83, 40, 16, 2, TFT_GREYOUT); // Vfo
  tft.drawRoundRect(48, 66, 23, 16, 2, TFT_GREYOUT); // M1
  tft.setTextColor(TFT_GREYOUT);
  tft.drawString("M1", 52, 66, 2);
  tft.drawString("HOME", 10, 66, 2);
  tft.drawString("VFO", 14, 83, 2);
  tft.drawRoundRect(48, 83, 23, 16, 2, TFT_WHITE); // M2
  tft.setTextColor(TFT_WHITE);
  tft.drawString("M2", 52, 83, 2);
}


void showVFO() {

  tft.drawRoundRect(48, 66, 23, 16, 2, TFT_GREYOUT); // M1
  tft.drawRoundRect(48, 83, 23, 16, 2, TFT_GREYOUT); // M2
  tft.drawRoundRect(6, 66, 40, 16, 2, TFT_GREYOUT); // Home
  tft.setTextColor(TFT_GREYOUT);
  tft.drawString("HOME", 10, 66, 2);
  tft.drawString("M1", 52, 66, 2);
  tft.drawString("M2", 52, 83, 2);
  tft.drawRoundRect(6, 83, 40, 16, 2, TFT_WHITE); // Vfo
  tft.setTextColor(TFT_WHITE);
  tft.drawString("VFO", 14, 83, 2);

}


void txStatus() {
  if (TXState >= 1) {

    Serial.println("TXaan");
    analogWrite(TXPin, TXPOWER * 2 + 12);
    TXPOWERON = TXPOWER * 2 + 55;
    tft.drawRoundRect(6, 34, 66, 31, 2, TFT_WHITE);
    tft.fillRoundRect(7, 35, 64, 29, 1, TFT_RED); // TXon
    tft.setTextColor(TFT_WHITE, TFT_RED);
    tft.drawString("TXon", 10, 38, 4);


    ShowPowerLevel();

  } else if (TXState <= 0) {

    Serial.println("TXuit");
    analogWrite(TXPin, 0);
    TXPOWERON = TXPOWER;
    tft.drawRoundRect(6, 34, 66, 31, 2, TFT_WHITE);
    tft.fillRoundRect(7, 35, 64, 29, 1, TFT_BLUE); // TXoff
    tft.setTextColor(TFT_WHITE, TFT_BLUE);
    tft.drawString("TXoff", 10, 38, 4);



    ShowPowerLevel();

  }
  Serial.println(TXState);
}
